@@ -23,7 +23,6 @@
 
 package ch.kuramo.javie.app;
 
-import java.io.PrintStream;
 import java.net.URL;
 
 import org.eclipse.core.runtime.FileLocator;
@@ -37,11 +36,6 @@ import org.eclipse.swt.SWTError;
 import org.eclipse.swt.graphics.RGB;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.ui.PlatformUI;
-import org.eclipse.ui.console.ConsolePlugin;
-import org.eclipse.ui.console.IConsole;
-import org.eclipse.ui.console.IConsoleManager;
-import org.eclipse.ui.console.IOConsoleOutputStream;
-import org.eclipse.ui.console.MessageConsole;
 import org.eclipse.ui.plugin.AbstractUIPlugin;
 import org.osgi.framework.BundleContext;
 
@@ -68,22 +62,6 @@ public class Activator extends AbstractUIPlugin {
 	public Activator() {
 	}
 
-	private void createDefaultConsole() {
-		MessageConsole console = new MessageConsole("コンソール", null);
-		IConsoleManager consoleManager = ConsolePlugin.getDefault().getConsoleManager();
-		consoleManager.addConsoles(new IConsole[] { console });
-
-		IOConsoleOutputStream out = console.newOutputStream();
-		IOConsoleOutputStream err = console.newOutputStream();
-
-		System.setOut(new PrintStream(out));
-		System.setErr(new PrintStream(err));
-
-		Display display = Display.getDefault();
-		//console.setBackground(display.getSystemColor(SWT.COLOR_GRAY));
-		err.setColor(display.getSystemColor(SWT.COLOR_RED));
-	}
-
 	/*
 	 * (non-Javadoc)
 	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
@@ -101,8 +79,6 @@ public class Activator extends AbstractUIPlugin {
 		DropTargetEnhancement.patch();
 		TreeViewerEditorEnhancement.patch();
 		ColorDialogBugFix.patch();
-
-		createDefaultConsole();
 	}
 
 	/*
