@@ -26,6 +26,12 @@ package ch.kuramo.javie.app.player;
 import org.eclipse.core.runtime.ListenerList;
 import org.eclipse.jface.resource.ImageRegistry;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.events.FocusAdapter;
+import org.eclipse.swt.events.FocusEvent;
+import org.eclipse.swt.events.KeyAdapter;
+import org.eclipse.swt.events.KeyEvent;
+import org.eclipse.swt.events.MouseAdapter;
+import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Image;
@@ -90,6 +96,10 @@ public class MediaPlayer extends Composite implements PlayerThreadListener, Play
 
 	private Time _frameDuration;
 
+	private boolean _scrubbing;
+
+	private boolean _audioScrubEnabled = true;
+
 	private volatile boolean _hidden = true;
 
 	private volatile boolean _dirty;
@@ -270,9 +280,62 @@ public class MediaPlayer extends Composite implements PlayerThreadListener, Play
 			_scale.setPageIncrement(nFrames / 10);
 			_scale.addSelectionListener(new SelectionAdapter() {
 				public void widgetSelected(SelectionEvent e) {
+					if (!_timeKeeper.isPaused()) {
+						_scrubbing = true;
+						pause();
+					}
 					setFrameNumber(_scale.getSelection());
 				}
 			});
+
+			_scale.addMouseListener(new MouseAdapter() {
+				public void mouseUp(MouseEvent e) {
+					if (_scrubbing) {
+						_scrubbing = false;
+						resume();
+					}
+				}
+			});
+
+			_scale.addKeyListener(new KeyAdapter() {
+				private boolean keyScrubbing;
+
+				public void keyPressed(KeyEvent e) {
+					switch (e.keyCode) {
+						case SWT.ARROW_LEFT:
+						case SWT.ARROW_RIGHT:
+						case SWT.ARROW_UP:
+						case SWT.ARROW_DOWN:
+							if (!_scrubbing && !_timeKeeper.isPaused()) {
+								keyScrubbing = true;
+							}
+							break;
+
+						// Macの場合はスペースキーで SelectionListener の widgetSelected が呼ばれてしまう。
+						// 以下はそれを回避するための措置。スペースキー以外にも同様のケースがあるかもしれないので default で処理している。
+						default:
+							e.doit = false;
+							break;
+					}
+				}
+
+				public void keyReleased(KeyEvent e) {
+					if (_scrubbing && keyScrubbing) {
+						_scrubbing = false;
+						keyScrubbing = false;
+						resume();
+					}
+				}
+			});
+
+			_scale.addFocusListener(new FocusAdapter() {
+				public void focusLost(FocusEvent e) {
+					if (_scrubbing) {
+						_scrubbing = false;
+						resume();
+					}
+				}
+			});
 		}
 
 		if (_mediaItem instanceof CompositionItem) {
@@ -297,8 +360,12 @@ public class MediaPlayer extends Composite implements PlayerThreadListener, Play
 
 		Time time = Time.fromFrameNumber(frameNumber, _frameDuration);
 		_timeKeeper.setTime(time);
-		if (_audioThread != null && !_timeKeeper.isPaused()) {
-			_audioThread.setTime(time);
+		if (_audioThread != null) {
+			if (!_timeKeeper.isPaused()) {
+				_audioThread.setTime(time);
+			} else if (_audioScrubEnabled) {
+				_audioThread.scrub(time);
+			}
 		}
 		if (_videoCanvas != null && _timeKeeper.isPaused()) {
 			_videoCanvas.forceRender();
@@ -417,6 +484,14 @@ public class MediaPlayer extends Composite implements PlayerThreadListener, Play
 		}
 	}
 
+	public boolean isAudioScrubEnabled() {
+		return _audioScrubEnabled;
+	}
+
+	public void setAudioScrubEnabled(boolean audioScrubEnabled) {
+		_audioScrubEnabled = audioScrubEnabled;
+	}
+
 	public boolean isShowInfo() {
 		return (_videoCanvas != null) ? _videoCanvas.isShowInfo() : false;
 	}
@@ -463,13 +538,36 @@ public class MediaPlayer extends Composite implements PlayerThreadListener, Play
 	public void handlePlayerLinkEvent(PlayerLinkEvent event) {
 		Time time = event.time;
 
-		_timeKeeper.setTime(time);
 		if (_hidden) {
-			_dirty = true;
+			_timeKeeper.setTime(time);
 			notifyPlayerTime(time);
+			_dirty = true;
 		} else {
-			if (_audioThread != null && !_timeKeeper.isPaused()) {
-				_audioThread.setTime(time);
+			if (event.isScrubbing()) {
+				if (!_timeKeeper.isPaused()) {
+					_scrubbing = true;
+					pause();
+				}
+			} else if (event.isEndScrubbing()) {
+				if (_scrubbing) {
+					_scrubbing = false;
+					resume();
+				}
+			} else if (_scrubbing) {
+				// ここに来ることはあるのだろうか？ あるなら何か見落としてる。
+				_logger.warn("event is neither scrubbing nor endScrubbing, but scrubbing flag of player is true.");
+				_scrubbing = false;
+			}
+
+			Time oldTime = _timeKeeper.getTime();
+			_timeKeeper.setTime(time);
+
+			if (_audioThread != null) {
+				if (!_timeKeeper.isPaused()) {
+					_audioThread.setTime(time);
+				} else if (_audioScrubEnabled && !time.equals(oldTime)) {
+					_audioThread.scrub(time);
+				}
 			}
 			if (_videoCanvas != null && _timeKeeper.isPaused()) {
 				_videoCanvas.forceRender();
