@@ -31,21 +31,15 @@ import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
+import org.mozilla.javascript.ContextFactory;
 
-import ch.kuramo.javie.api.Size2i;
-import ch.kuramo.javie.api.Vec2d;
-import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.app.InjectorHolder;
-import ch.kuramo.javie.core.AnimatableVec2d;
-import ch.kuramo.javie.core.Composition;
+import ch.kuramo.javie.core.AnimatableValue;
 import ch.kuramo.javie.core.Effect;
 import ch.kuramo.javie.core.EffectDescriptor;
 import ch.kuramo.javie.core.EffectableLayer;
 import ch.kuramo.javie.core.JavieRuntimeException;
 import ch.kuramo.javie.core.LayerComposition;
-import ch.kuramo.javie.core.LayerNature;
-import ch.kuramo.javie.core.MediaInput;
-import ch.kuramo.javie.core.MediaLayer;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.PropertyDescriptor;
@@ -85,6 +79,9 @@ public class AddEffectsOperation extends ProjectOperation {
 	@Inject
 	private ProjectDecoder _decoder;
 
+	@Inject
+	private ContextFactory _jsContextFactory;
+
 
 	public AddEffectsOperation(
 			ProjectManager projectManager, Map<EffectableLayer, Integer> effectIndices, EffectDescriptor ed) {
@@ -113,36 +110,16 @@ public class AddEffectsOperation extends ProjectOperation {
 			effectIndex = Math.max(0, Math.min(effectIndex, layer.getEffects().size()));
 
 			Effect effect = _effectRegistry.newEffect(ed.getType());
-			centerProperties(effect, layer, comp);
+			resetEffectProperties(comp, layer, effect);
 			String data = _encoder.encodeElement(effect);
 			_entries.add(new Entry(layer.getId(), data, effectIndex));
 		}
 	}
 
-	private void centerProperties(Effect effect, EffectableLayer layer, Composition comp) {
-		Vec2d center = null;
-		if (LayerNature.isCTCR(layer)) {
-			Size2i size = comp.getSize();
-			center = new Vec2d(size.width/2.0, size.height/2.0);
-
-		} else if (layer instanceof MediaLayer) {
-			MediaInput input = ((MediaLayer) layer).getMediaInput();
-			if (input != null) {
-				VideoBounds bounds = input.getVideoFrameBounds();
-				if (bounds != null) {
-					center = new Vec2d(bounds.x + bounds.width/2.0, bounds.y + bounds.height/2.0);
-				}
-			}
-		}
-
-		if (center != null) {
-			for (PropertyDescriptor pd : effect.getEffectDescriptor().getPropertyDescriptors()) {
-				if (pd.getPropertyClass() == AnimatableVec2d.class
-						&& Boolean.parseBoolean(pd.getString("initiallyCenter"))) {
-					AnimatableVec2d avalue = (AnimatableVec2d) pd.get(effect);
-					avalue.reset(center);
-				}
-			}
+	private void resetEffectProperties(LayerComposition comp, EffectableLayer layer, Effect effect) {
+		for (PropertyDescriptor pd : effect.getEffectDescriptor().getPropertyDescriptors()) {
+			AnimatableValue<?> avalue = (AnimatableValue<?>) pd.get(effect);
+			ResetAnimatableValuesOperation.resetEffectProperty(comp, layer, pd, avalue, _jsContextFactory);
 		}
 	}
 
