@@ -23,6 +23,9 @@
 
 package ch.kuramo.javie.app.project;
 
+import java.util.Collection;
+import java.util.Collections;
+import java.util.Map;
 import java.util.SortedMap;
 
 import org.eclipse.core.commands.ExecutionException;
@@ -40,6 +43,7 @@ import ch.kuramo.javie.core.Interpolation;
 import ch.kuramo.javie.core.Keyframe;
 import ch.kuramo.javie.core.LayerComposition;
 import ch.kuramo.javie.core.Project;
+import ch.kuramo.javie.core.Util;
 
 public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 
@@ -51,7 +55,7 @@ public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 
 	private final String _property;
 
-	private final Keyframe<V> _oldKeyframe;
+	private final Collection<Keyframe<V>> _oldKeyframes;
 
 	private volatile Keyframe<V> _newKeyframe;
 
@@ -65,18 +69,37 @@ public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 		LayerComposition comp = projectManager.checkLayer(layer);
 		Effect effect = layer.getEffects().get(effectIndex);
 		AnimatableValue<V> avalue = PropertyUtil.getProperty(effect, property);
-		Keyframe<V> oldKeyframe = avalue.getKeyframe(time);
 
-		setLabel(avalue.hasKeyframe(), (oldKeyframe != null), (value != null));
+		Time frameDuration = comp.getFrameDuration();
+		Time halfOfFrameDuration = new Time(frameDuration.timeValue/2, frameDuration.timeScale);
+		Time t1 = time.subtract(halfOfFrameDuration);
+		Time t2 = time.add(frameDuration).subtract(halfOfFrameDuration);
+		SortedMap<Time, Keyframe<V>> oldKeyframeMap = avalue.getKeyframeMap().subMap(t1, t2);
+
+		setLabel(avalue.hasKeyframe(), !oldKeyframeMap.isEmpty(), (value != null));
 
 		_compId = comp.getId();
 		_layerId = layer.getId();
 		_effectIndex = effectIndex;
 		_property = property;
-		_oldKeyframe = oldKeyframe;
+		_oldKeyframes = Util.newSet(oldKeyframeMap.values());
 
 		if (value != null) {
-			_newKeyframe = new Keyframe<V>(time, value, defaultInterpolation(avalue, oldKeyframe, time));
+			Keyframe<V> kf = oldKeyframeMap.get(time);
+			if (kf == null) {
+				SortedMap<Time, Keyframe<V>> head = oldKeyframeMap.headMap(time);
+				SortedMap<Time, Keyframe<V>> tail = oldKeyframeMap.tailMap(time);
+				Keyframe<V> kf1 = head.isEmpty() ? null : head.get(head.lastKey());
+				Keyframe<V> kf2 = tail.isEmpty() ? null : tail.get(tail.firstKey());
+				if (kf1 != null && kf2 != null) {
+					kf = kf2.time.subtract(time).before(time.subtract(kf1.time)) ? kf2 : kf1;
+				} else if (kf1 != null) {
+					kf = kf1;
+				} else if (kf2 != null) {
+					kf = kf2;
+				}
+			}
+			_newKeyframe = new Keyframe<V>(time, value, defaultInterpolation(avalue, kf, time));
 		} else {
 			_newKeyframe = null;
 		}
@@ -102,7 +125,7 @@ public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 		if (oldKeyframe != null) {
 			return oldKeyframe.interpolation;
 		} else {
-			SortedMap<Time, Keyframe<V>> head = avalue.headKeyframeMap(time);
+			SortedMap<Time, Keyframe<V>> head = avalue.getKeyframeMap().headMap(time);
 			if (!head.isEmpty()) {
 				return head.get(head.lastKey()).interpolation;
 			} else {
@@ -115,7 +138,7 @@ public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 	protected IStatus executeOrRedo(IProgressMonitor monitor, IAdaptable info,
 			Project project, ProjectManager pm) throws ExecutionException {
 
-		replaceKeyframe(_oldKeyframe, _newKeyframe, project, pm);
+		replaceKeyframes(_oldKeyframes, Collections.singleton(_newKeyframe), project, pm);
 		return Status.OK_STATUS;
 	}
 
@@ -123,30 +146,40 @@ public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 	protected IStatus undo(IProgressMonitor monitor, IAdaptable info,
 			Project project, ProjectManager pm) throws ExecutionException {
 
-		replaceKeyframe(_newKeyframe, _oldKeyframe, project, pm);
+		replaceKeyframes(Collections.singleton(_newKeyframe), _oldKeyframes, project, pm);
 		return Status.OK_STATUS;
 	}
 
-	private void replaceKeyframe(
-			Keyframe<V> target, Keyframe<V> replacement,
+	private void replaceKeyframes(
+			Collection<Keyframe<V>> target, Collection<Keyframe<V>> replacement,
 			Project project, ProjectManager pm) {
 
 		LayerComposition comp = project.getComposition(_compId);
 		EffectableLayer layer = (EffectableLayer) comp.getLayer(_layerId);
 		Effect effect = layer.getEffects().get(_effectIndex);
 		AnimatableValue<V> avalue = PropertyUtil.getProperty(effect, _property);
-		Time time;
 
-		if (replacement != null) {
-			avalue.putKeyframe(replacement);
-			time = replacement.time;
-		} else {
-			avalue.removeKeyframe(target.time);
-			time = target.time;
+		Map<Time, Object[]> data = null;
+		if (pm != null) {
+			data = Util.newMap();
+		}
+
+		for (Keyframe<V> kf : target) {
+			avalue.removeKeyframe(kf.time);
+			if (pm != null) {
+				data.put(kf.time, new Object[] { layer, _effectIndex, _property, kf.time });
+			}
+		}
+
+		for (Keyframe<V> kf : replacement) {
+			avalue.putKeyframe(kf);
+			if (pm != null) {
+				data.put(kf.time, new Object[] { layer, _effectIndex, _property, kf.time });
+			}
 		}
 
 		if (pm != null) {
-			pm.fireKeyframesChange(comp, new Object[][] { new Object[] { layer, _effectIndex, _property, time } });
+			pm.fireKeyframesChange(comp, data.values().toArray(new Object[data.size()][]));
 		}
 	}
 
@@ -158,7 +191,7 @@ public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 			// _newKeyframe の値は、シャドウプロジェクトに反映される前に
 			// 後続のマージ可能オペレーションで上書きされているかもしれない。
 			// しかし、シャドウオペレーションのキューが全て実行された時点での値は一致するので問題ない。
-			replaceKeyframe(null, _newKeyframe, project, pm);
+			replaceKeyframes(Collections.<Keyframe<V>>emptySet(), Collections.singleton(_newKeyframe), project, pm);
 			return true;
 		}
 
@@ -188,10 +221,15 @@ public class ModifyEffectKeyframeOperation<V> extends ProjectOperation {
 
 		if (!_newKeyframe.value.equals(newKeyframe.value)) {
 			_newKeyframe = newKeyframe;
-			replaceKeyframe(null, _newKeyframe, project, pm);
+			replaceKeyframes(Collections.<Keyframe<V>>emptySet(), Collections.singleton(_newKeyframe), project, pm);
 
 			// 結合の結果、このオペレーションの直前の値に戻ったかどうか。
-			_noEffect = (_oldKeyframe != null && _oldKeyframe.value.equals(_newKeyframe.value));
+			boolean noEffect = false;
+			if (_oldKeyframes.size() == 1) {
+				Keyframe<V> oldKeyframe = _oldKeyframes.iterator().next();
+				noEffect = (oldKeyframe.time.equals(_newKeyframe.time) && oldKeyframe.value.equals(_newKeyframe.value));
+			}
+			_noEffect = noEffect;
 		}
 
 		return true;
