@@ -777,7 +777,7 @@ public class LayerCompositionView extends ViewPart
 						time = _composition.getDuration().subtract(frameDuration);
 					}
 					time = Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
-					update(time);
+					update(time, null);
 				}
 			}
 		});
@@ -913,7 +913,7 @@ public class LayerCompositionView extends ViewPart
 										}
 										newTime[0] = Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
 
-										update(newTime[0]);
+										update(newTime[0], Boolean.TRUE);
 										_timeCodeComposite.redraw();
 									}
 
@@ -923,6 +923,10 @@ public class LayerCompositionView extends ViewPart
 
 							case SWT.MouseUp:
 							case SWT.Deactivate:
+								if (newTime[0] != null /* && !_timeCodeComposite.isDisposed() */) {		// TODO disposeされてるかどうかのチェックは必要？
+									update(newTime[0], Boolean.FALSE);									//      必要だとしても、スクラブ終了のイベントが送出されなくなるのはまずい。
+								}
+
 								filter = null;
 
 								display.removeFilter(SWT.MouseMove, this);
@@ -2752,7 +2756,7 @@ public class LayerCompositionView extends ViewPart
 		_treeViewer.editElement(exprElement, LayerCompositionView.TIMELINE_COL);
 	}
 
-	public void update(Time time) {
+	public void update(Time time, Boolean scrubbing) {
 		if (_playerLinkListeners.isEmpty()) {
 			_time = time;
 			collectAnimatableValues(time);
@@ -2762,7 +2766,7 @@ public class LayerCompositionView extends ViewPart
 
 		} else {
 			collectAnimatableValues(time);
-			refreshPlayer(time);
+			refreshPlayer(time, scrubbing);
 		}
 	}
 
@@ -2777,8 +2781,8 @@ public class LayerCompositionView extends ViewPart
 		_playerLinkListeners.remove(listener);
 	}
 
-	private void refreshPlayer(Time time) {
-		PlayerLinkEvent event = new PlayerLinkEvent(this, time);
+	private void refreshPlayer(Time time, Boolean scrubbing) {
+		PlayerLinkEvent event = new PlayerLinkEvent(this, time, scrubbing);
 		for (Object l : _playerLinkListeners.getListeners()) {
 			((PlayerLinkListener) l).handlePlayerLinkEvent(event);
 		}
