@@ -23,10 +23,7 @@
 
 package ch.kuramo.javie.app.views.layercomp;
 
-import java.lang.reflect.InvocationTargetException;
-import java.util.Locale;
-import java.util.MissingResourceException;
-import java.util.ResourceBundle;
+import java.util.List;
 
 import org.eclipse.jface.viewers.ArrayContentProvider;
 import org.eclipse.jface.viewers.CellEditor;
@@ -37,10 +34,10 @@ import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.widgets.Combo;
 
+import ch.kuramo.javie.app.EnumLabels;
 import ch.kuramo.javie.app.views.ComboBoxViewerCellEditor;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.core.AnimatableEnum;
-import ch.kuramo.javie.core.JavieRuntimeException;
 
 public class AnimatableEnumElementDelegate<E extends Enum<E>> extends AnimatableValueElementDelegate<E> {
 
@@ -48,7 +45,7 @@ public class AnimatableEnumElementDelegate<E extends Enum<E>> extends Animatable
 
 	private final E[] values;
 
-	private final String[] texts;
+	private final List<String> labels;
 
 	private ComboBoxViewerCellEditor editor;
 
@@ -58,12 +55,12 @@ public class AnimatableEnumElementDelegate<E extends Enum<E>> extends Animatable
 
 		super(element, name, avalue);
 		enumType = avalue.getEnumType();
-		values = values();
-		texts = texts();
+		values = enumType.getEnumConstants();
+		labels = EnumLabels.getLabels(enumType);
 	}
 
 	protected void drawValue(GC gc, int x, int y, int height, boolean focused) {
-		drawValue(gc, x, y, height, focused, texts[value.ordinal()]);
+		drawValue(gc, x, y, height, focused, labels.get(value.ordinal()));
 	}
 
 	protected boolean canDragGestureEdit() {
@@ -97,7 +94,7 @@ public class AnimatableEnumElementDelegate<E extends Enum<E>> extends Animatable
 
 			editor.setInput(null);
 			editor.setLabelProvider(new EnumLabelProvider());
-			editor.setInput(values());
+			editor.setInput(values);
 
 			return editor;
 		}
@@ -117,54 +114,9 @@ public class AnimatableEnumElementDelegate<E extends Enum<E>> extends Animatable
 		}
 	}
 
-	private E[] values() {
-		try {
-			@SuppressWarnings("unchecked")
-			E[] values = (E[]) enumType.getMethod("values").invoke(null);
-			return values;
-		} catch (IllegalAccessException e) {
-			throw new JavieRuntimeException(e);
-		} catch (InvocationTargetException e) {
-			throw new JavieRuntimeException(e);
-		} catch (NoSuchMethodException e) {
-			throw new JavieRuntimeException(e);
-		}
-	}
-
-	private String[] texts() {
-		String[] texts = new String[values.length];
-		for (int i = 0; i < values.length; ++i) {
-			texts[i] = '!' + values[i].name() + '!';
-		}
-
-		ResourceBundle bundle;
-		try {
-			bundle = ResourceBundle.getBundle(
-					enumType.getPackage().getName() + ".messages",
-					Locale.getDefault(), enumType.getClassLoader());
-		} catch (MissingResourceException e) {
-			return texts;
-		}
-
-		for (int i = 0; i < values.length; ++i) {
-			StringBuilder sb = new StringBuilder();
-			for (Class<?> c = enumType; c != null; c = c.getEnclosingClass()) {
-				sb.insert(0, c.getSimpleName());
-				try {
-					texts[i] = bundle.getString(String.format(
-							"%s.%s", sb.toString(), values[i].name()));
-				} catch (MissingResourceException e) {
-					// ignore
-				}
-				sb.insert(0, '$');
-			}
-		}
-		return texts;
-	}
-
 	private class EnumLabelProvider extends LabelProvider {
 		public String getText(Object element) {
-			return texts[enumType.cast(element).ordinal()];
+			return labels.get(enumType.cast(element).ordinal());
 		}
 	}
 
