@@ -57,6 +57,10 @@ public class GridBuilder {
 		return _tabList.toArray(new Control[_tabList.size()]);
 	}
 
+	public GridLayout getLayout() {
+		return (GridLayout) _composite.getLayout();
+	}
+
 	public CellBuilder span(int hSpan, int vSpan) {
 		return new CellBuilder().span(hSpan, vSpan);
 	}
@@ -249,7 +253,9 @@ public class GridBuilder {
 
 		public Group group(int style, String text) {
 			Group group = new Group(_composite, style);
-			group.setText(text);
+			if (text != null) {
+				group.setText(text);
+			}
 			group.setLayoutData(data);
 			addToTabList(group);
 			return group;
