@@ -23,10 +23,11 @@
 
 package ch.kuramo.javie.core;
 
+import ch.kuramo.javie.api.ICamera;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Vec3d;
 
-public interface Camera {
+public interface Camera extends ICamera {
 
 	Size2i getViewportSize();
 
