@@ -172,32 +172,27 @@ public class LayerNature {
 		if (layer instanceof MediaItemLayer) {
 			MediaItemLayer miLayer = (MediaItemLayer) layer;
 			MediaInput input = miLayer.getMediaInput();
-			return (input instanceof MediaFileInput && input.isVideoAvailable() && input.getVideoFrameDuration() != null);
+			return (!miLayer.isPrecompositionLayer()
+					&& input.isVideoAvailable()
+					&& input.getVideoFrameDuration() != null
+					&& input.getVideoFrameDuration().timeValue > 0);
 		}
 
 		return false;
 	}
 
 	public static FrameBlend getFrameBlend(Layer layer) {
-		if (layer instanceof MediaItemLayer) {
-			MediaItemLayer miLayer = (MediaItemLayer) layer;
-			MediaInput input = miLayer.getMediaInput();
-			if (input instanceof MediaFileInput && input.isVideoAvailable() && input.getVideoFrameDuration() != null) {
-				return miLayer.getFrameBlend();
-			}
+		if (isFrameBlendNature(layer)) {
+			return ((MediaItemLayer) layer).getFrameBlend();
 		}
 
 		return FrameBlend.NONE;
 	}
 
 	public static void setFrameBlend(Layer layer, FrameBlend frameBlend) {
-		if (layer instanceof MediaItemLayer) {
-			MediaItemLayer miLayer = (MediaItemLayer) layer;
-			MediaInput input = miLayer.getMediaInput();
-			if (input instanceof MediaFileInput && input.isVideoAvailable() && input.getVideoFrameDuration() != null) {
-				miLayer.setFrameBlend(frameBlend);
-				return;
-			}
+		if (isFrameBlendNature(layer)) {
+			((MediaItemLayer) layer).setFrameBlend(frameBlend);
+			return;
 		}
 
 		throw new UnsupportedOperationException(); 
@@ -289,14 +284,8 @@ public class LayerNature {
 	}
 
 	public static boolean isTimeRemapEnabled(Layer layer) {
-		if (layer instanceof MediaLayer) {
-			MediaLayer mediaLayer = (MediaLayer) layer;
-			MediaInput input = mediaLayer.getMediaInput();
-			return (input != null && input.getDuration() != null
-					&& mediaLayer.getTimeRemap().hasKeyframe());
-		}
-
-		return false;
+		return isTimeRemapNature(layer)
+			&& ((MediaLayer) layer).getTimeRemap().hasKeyframe();
 	}
 
 
