@@ -24,17 +24,19 @@
 package ch.kuramo.javie.core.internal;
 
 import net.arnx.jsonic.JSONHint;
+import ch.kuramo.javie.api.IAudioBuffer;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.core.AbstractItem;
-import ch.kuramo.javie.core.AudioBuffer;
 import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.CompositionItem;
 import ch.kuramo.javie.core.MediaInput;
+import ch.kuramo.javie.core.MediaOptions;
+import ch.kuramo.javie.core.MediaSource;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.WrappedOperation;
 import ch.kuramo.javie.core.annotations.ProjectElement;
 import ch.kuramo.javie.core.services.RenderContext;
@@ -96,6 +98,21 @@ public class CompositionItemImpl extends AbstractItem implements CompositionItem
 		}
 	}
 
+	@JSONHint(ignore=true)
+	public MediaSource getMediaSource() {
+		return null;
+	}
+
+	@JSONHint(ignore=true)
+	public MediaOptions getMediaOptions() {
+		return null;
+	}
+
+	@JSONHint(ignore=true)
+	public void setMediaOptions(MediaOptions options) {
+		throw new UnsupportedOperationException();
+	}
+
 	@JSONHint(ignore=true)
 	public MediaInput getMediaInput() {
 		return _input;
@@ -109,10 +126,6 @@ public class CompositionItemImpl extends AbstractItem implements CompositionItem
 
 	private class CompositionInput implements MediaInput {
 
-		public void dispose() {
-			// noting to do
-		}
-
 		public boolean isVideoAvailable() {
 			// TODO コンポジションがビデオを持っている場合だけtrueを返すようにする。
 			return true;
@@ -136,9 +149,9 @@ public class CompositionItemImpl extends AbstractItem implements CompositionItem
 			return new VideoBounds(size.width, size.height);  // TODO 毎回newするのは無駄
 		}
 
-		public VideoBuffer getVideoFrame(final Time mediaTime) {
-			VideoBuffer vb = _context.saveAndExecute(new WrappedOperation<VideoBuffer>() {
-				public VideoBuffer execute() {
+		public IVideoBuffer getVideoFrame(final Time mediaTime) {
+			IVideoBuffer vb = _context.saveAndExecute(new WrappedOperation<IVideoBuffer>() {
+				public IVideoBuffer execute() {
 					_context.setTime(mediaTime);
 					return _composition.renderVideoFrame();
 				}
@@ -147,9 +160,9 @@ public class CompositionItemImpl extends AbstractItem implements CompositionItem
 			return vb;
 		}
 
-		public AudioBuffer getAudioChunk(final Time mediaTime) {
-			return _context.saveAndExecute(new WrappedOperation<AudioBuffer>() {
-				public AudioBuffer execute() {
+		public IAudioBuffer getAudioChunk(final Time mediaTime) {
+			return _context.saveAndExecute(new WrappedOperation<IAudioBuffer>() {
+				public IAudioBuffer execute() {
 					_context.setTime(mediaTime);
 					return _composition.renderAudioChunk();
 				}
