@@ -27,13 +27,13 @@ import java.util.List;
 import java.util.ListIterator;
 
 import ch.kuramo.javie.api.ColorMode;
+import ch.kuramo.javie.api.IAudioBuffer;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.api.Vec3d;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.core.AbstractComposition;
-import ch.kuramo.javie.core.AudioBuffer;
-import ch.kuramo.javie.core.Camera;
 import ch.kuramo.javie.core.CameraLayer;
 import ch.kuramo.javie.core.CollapseTransformation;
 import ch.kuramo.javie.core.CoreContext;
@@ -48,7 +48,6 @@ import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.TrackMatte;
 import ch.kuramo.javie.core.Util;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.VideoLayerBuffer;
 import ch.kuramo.javie.core.VideoLayerComposer;
 import ch.kuramo.javie.core.VideoLayerRenderer;
@@ -186,7 +185,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 		}
 	}
 
-	public VideoBuffer renderVideoFrame() {
+	public IVideoBuffer renderVideoFrame() {
 		_context.setComposition(this);
 
 		ColorMode colorMode = getColorMode();
@@ -214,9 +213,11 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 
 		Size2i renderSize = _context.getVideoResolution().scale(getSize());
 
-		final CameraImpl camera = new CameraImpl(
+		CameraImpl camera = new CameraImpl(
 				_context, _vrSupport, cameraLayer, getSize(), renderSize);
 
+		_context.setCamera(camera);
+
 		final VideoLayerComposerImpl composer = new VideoLayerComposerImpl(
 				_context, _vrSupport, colorMode, renderSize, camera.getPosition());
 
@@ -239,14 +240,14 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 				if (ml.getTrackMatte() != TrackMatte.NONE && i+1 < n) {
 					Layer matteLayer = _layers.get(i+1);
 					if (matteLayer instanceof MediaLayer) {
-						setupTrackMatteRenderer(ml, (MediaLayer) matteLayer, composer, camera, null, motionBlur);
+						setupTrackMatteRenderer(ml, (MediaLayer) matteLayer, composer, null, motionBlur);
 						continue;
 					}
 				}
 
 				_context.saveAndExecute(new WrappedOperation<Object>() {
 					public Object execute() {
-						ml.setupVideoRenderer(composer, camera, null, motionBlur, _frameBlendEnabled, true, true);
+						ml.setupVideoRenderer(composer, null, motionBlur, _frameBlendEnabled, true, true);
 						return null;
 					}
 				});
@@ -257,7 +258,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 	}
 
 	public void setupCollapseTransformation(
-			final VideoLayerComposer composer, final Camera camera, final CollapseTransformation ct, final MotionBlur motionBlur) {
+			final VideoLayerComposer composer, final CollapseTransformation ct, final MotionBlur motionBlur) {
 
 		_context.setComposition(this);
 
@@ -280,14 +281,14 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 				if (ml.getTrackMatte() != TrackMatte.NONE && i+1 < n) {
 					Layer matteLayer = _layers.get(i+1);
 					if (matteLayer instanceof MediaLayer) {
-						setupTrackMatteRenderer(ml, (MediaLayer) matteLayer, composer, camera, ct, motionBlur);
+						setupTrackMatteRenderer(ml, (MediaLayer) matteLayer, composer, ct, motionBlur);
 						continue;
 					}
 				}
 
 				_context.saveAndExecute(new WrappedOperation<Object>() {
 					public Object execute() {
-						ml.setupVideoRenderer(composer, camera, ct, motionBlur, _frameBlendEnabled, true, true);
+						ml.setupVideoRenderer(composer, ct, motionBlur, _frameBlendEnabled, true, true);
 						return null;
 					}
 				});
@@ -297,7 +298,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 
 	private void setupTrackMatteRenderer(
 			final MediaLayer fillLayer, final MediaLayer matteLayer,
-			VideoLayerComposer composer, final Camera camera, final CollapseTransformation ct, final MotionBlur motionBlur) {
+			VideoLayerComposer composer, final CollapseTransformation ct, final MotionBlur motionBlur) {
 
 		final VideoLayerRenderer[] renderers = new VideoLayerRenderer[2];
 		final Vec3d[][] vertices = new Vec3d[1][];
@@ -308,7 +309,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 				fillLayer.setupVideoRenderer(new VideoLayerComposer() {
 					public void add2D(VideoLayerRenderer r)							{ renderers[0] = r; }
 					public void add3D(VideoLayerRenderer r, Vec3d[] v, String ig)	{ renderers[0] = r; vertices[0] = v; intersectionGroup[0] = ig; }
-				}, camera, ct, motionBlur, _frameBlendEnabled, true, true);
+				}, ct, motionBlur, _frameBlendEnabled, true, true);
 				return null;
 			}
 		});
@@ -318,7 +319,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 				matteLayer.setupVideoRenderer(new VideoLayerComposer() {
 					public void add2D(VideoLayerRenderer r)							{ renderers[1] = r; }
 					public void add3D(VideoLayerRenderer r, Vec3d[] v, String ig)	{ renderers[1] = r; }
-				}, camera, ct, motionBlur, _frameBlendEnabled, true, true);
+				}, ct, motionBlur, _frameBlendEnabled, true, true);
 				return null;
 			}
 		});
@@ -343,13 +344,13 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 			public VideoLayerBuffer render(boolean withDepthBuffer) {
 				VideoLayerBuffer fillBuffer = null;
 				VideoLayerBuffer matteBuffer = null;
-				VideoBuffer vbFill = null;
+				IVideoBuffer vbFill = null;
 				try {
 					fillBuffer = renderers[0].render(withDepthBuffer);
 					matteBuffer = renderers[1].render(false);
 					vbFill = fillBuffer.getVideoBuffer();
 
-					VideoBuffer resultBuffer = _vrSupport.createVideoBuffer(vbFill.getColorMode(), vbFill.getBounds());
+					IVideoBuffer resultBuffer = _vrSupport.createVideoBuffer(vbFill.getColorMode(), vbFill.getBounds());
 					_vrSupport.trackMatte(matteBuffer.getVideoBuffer(), fillBuffer.getVideoBuffer(), resultBuffer,
 							trackMatte, matteBuffer.getOpacity());
 
@@ -364,7 +365,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 						matteBuffer.dispose();
 					}
 
-					// 正常時はfillBufferを破棄してしまうとDepthBufferまで破棄されてしまうので、VideoBufferだけ破棄する。
+					// 正常時はfillBufferを破棄してしまうとDepthBufferまで破棄されてしまうので、IVideoBufferだけ破棄する。
 					// (tryブロックを抜ける直前にfillBufferにnullを代入しているので、ここでfillBufferがnullでないのは例外発生時のみ)
 					if (fillBuffer != null) {
 						fillBuffer.dispose();
@@ -382,7 +383,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 		}
 	}
 
-	public VideoBuffer getLayerVideoFrame(Layer layer, final boolean withEffects, final boolean withTransformation) {
+	public IVideoBuffer getLayerVideoFrame(Layer layer, final boolean withEffects, final boolean withTransformation) {
 		if (_context.getComposition() != this) {
 			throw new IllegalStateException("this composition is not the context composition.");
 		}
@@ -408,11 +409,11 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 			}
 
 			VideoBounds bounds = input.getVideoFrameBounds();
-			Size2i layerSize = LayerNature.isCTCR(layer) ? getSize() : new Size2i(bounds.width, bounds.height);
-			Size2i viewportSize = _context.getVideoResolution().scale(layerSize);
+			final Size2i layerSize = LayerNature.isCTCR(layer) ? getSize() : new Size2i(bounds.width, bounds.height);
+			final Size2i viewportSize = _context.getVideoResolution().scale(layerSize);
 
 			if (viewportSize.width == 0 || viewportSize.height == 0) {
-				VideoBuffer vb = _vrSupport.createVideoBuffer(_context.getColorMode(), viewportSize);
+				IVideoBuffer vb = _vrSupport.createVideoBuffer(_context.getColorMode(), viewportSize);
 				vb.clear();
 				return vb;
 			}
@@ -424,15 +425,15 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 				public void add3D(VideoLayerRenderer r, Vec3d[] v, String ig)	{ renderers[0] = r; }
 			};
 
-			final CameraImpl camera = new CameraImpl(
-					_context, _vrSupport, null, layerSize, viewportSize);
-
 			final MotionBlur motionBlur = _motionBlurEnabled ? _motionBlur : null;
 
 			_context.saveAndExecute(new WrappedOperation<Object>() {
 				public Object execute() {
+					_context.setCamera(new CameraImpl(
+							_context, _vrSupport, null, layerSize, viewportSize));
+
 					ml.setupVideoRenderer(
-							composer, camera, null, motionBlur, _frameBlendEnabled,
+							composer, null, motionBlur, _frameBlendEnabled,
 							withEffects, withTransformation);
 					return null;
 				}
@@ -445,14 +446,17 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 		return null;
 	}
 
-	public AudioBuffer renderAudioChunk() {
+	public IAudioBuffer renderAudioChunk() {
 		_context.setComposition(this);
 
+		ColorMode colorMode = getColorMode();
+		_context.setColorMode(colorMode);
+
 		if (isFrameDurationPreserved()) {
 			_context.setVideoFrameDuration(getFrameDuration());
 		}
 
-		AudioBuffer ab = null;
+		IAudioBuffer ab = null;
 		Time time = _context.getTime();
 		Time chunkDuration = Time.fromFrameNumber(
 				_context.getAudioFrameCount(), _context.getAudioMode().sampleDuration);
@@ -469,8 +473,8 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 			if (layer instanceof MediaLayer) {
 				final MediaLayer al = (MediaLayer) layer;
 
-				AudioBuffer alab = _context.saveAndExecute(new WrappedOperation<AudioBuffer>() {
-					public AudioBuffer execute() {
+				IAudioBuffer alab = _context.saveAndExecute(new WrappedOperation<IAudioBuffer>() {
+					public IAudioBuffer execute() {
 						return al.renderAudioChunk(true, true);
 					}
 				});
@@ -479,7 +483,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 					if (ab == null) {
 						ab = alab;
 					} else {
-						AudioBuffer abOld = ab;
+						IAudioBuffer abOld = ab;
 						try {
 							ab = _arSupport.mix(ab, alab);
 						} finally {
@@ -498,7 +502,7 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 		return ab;
 	}
 
-	public AudioBuffer getLayerAudioChunk(Layer layer, final boolean withEffects, final boolean withLevelsAndInOut) {
+	public IAudioBuffer getLayerAudioChunk(Layer layer, final boolean withEffects, final boolean withLevelsAndInOut) {
 		if (_context.getComposition() != this) {
 			throw new IllegalStateException("this composition is not the context composition.");
 		}
@@ -523,8 +527,8 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 				return null;
 			}
 
-			return _context.saveAndExecute(new WrappedOperation<AudioBuffer>() {
-				public AudioBuffer execute() {
+			return _context.saveAndExecute(new WrappedOperation<IAudioBuffer>() {
+				public IAudioBuffer execute() {
 					return ml.renderAudioChunk(withEffects, withLevelsAndInOut);
 				}
 			});
