@@ -26,17 +26,20 @@ package ch.kuramo.javie.core.internal;
 import java.io.File;
 
 import net.arnx.jsonic.JSONHint;
+import ch.kuramo.javie.api.IAudioBuffer;
+import ch.kuramo.javie.api.IVideoBuffer;
+import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.core.AbstractItem;
 import ch.kuramo.javie.core.FileItem;
-import ch.kuramo.javie.core.MediaFileInput;
 import ch.kuramo.javie.core.MediaInput;
-import ch.kuramo.javie.core.MediaInputPlaceholder;
 import ch.kuramo.javie.core.MediaItem;
+import ch.kuramo.javie.core.MediaOptions;
+import ch.kuramo.javie.core.MediaSource;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.annotations.ProjectElement;
-import ch.kuramo.javie.core.services.MediaFileInputFactory;
+import ch.kuramo.javie.core.services.MediaSourceFactory;
 
 import com.google.inject.Inject;
 import com.google.inject.Injector;
@@ -46,12 +49,16 @@ public class MediaFileItem extends AbstractItem implements MediaItem, FileItem {
 
 	private File _file;
 
-	private VideoBounds _videoFrameBounds;
+	private MediaSource _mediaSource;
+
+	private MediaOptions _mediaOptions;
 
 	private MediaInput _mediaInput;
 
+	private VideoBounds _placeholderBounds;
+
 	@Inject
-	private MediaFileInputFactory _mediaFileInputFactory;
+	private MediaSourceFactory _mediaSourceFactory;
 
 	@Inject
 	private Injector _injector;
@@ -64,13 +71,14 @@ public class MediaFileItem extends AbstractItem implements MediaItem, FileItem {
 	public void initialize(File file) {
 		super.initialize();
 
-		MediaFileInput input = _mediaFileInputFactory.get(getId(), file);
+		_mediaSource = _mediaSourceFactory.get(getId(), file);
 
-		if (input != null) {
+		if (_mediaSource != null) {
 			_file = file;
-			_videoFrameBounds = input.isVideoAvailable() ? input.getVideoFrameBounds() : null;
-			_mediaInput = input;
-			setName(_file.getName());
+			setName(file.getName());
+
+			_mediaOptions = _mediaSource.validateOptions(null);
+			_mediaInput = new MediaInputImpl();
 		}
 	}
 
@@ -81,11 +89,11 @@ public class MediaFileItem extends AbstractItem implements MediaItem, FileItem {
 	}
 
 	public void dispose() {
-		if (_mediaInput != null) {
-			if (!(_mediaInput instanceof MediaInputPlaceholder)) {
-				_mediaFileInputFactory.release(getId());
-			}
-			_mediaInput = null;
+		_mediaInput = null;
+
+		if (_mediaSource != null) {
+			_mediaSourceFactory.release(getId());
+			_mediaSource = null;
 		}
 	}
 
@@ -97,12 +105,27 @@ public class MediaFileItem extends AbstractItem implements MediaItem, FileItem {
 		_file = file;
 	}
 
-	public VideoBounds getVideoFrameBounds() {
-		return _videoFrameBounds;
+	@JSONHint(ignore=true)
+	public MediaSource getMediaSource() {
+		return _mediaSource;
 	}
 
-	public void setVideoFrameBounds(VideoBounds videoFrameBounds) {
-		_videoFrameBounds = videoFrameBounds;
+	public MediaOptions getMediaOptions() {
+		return (_mediaOptions != null) ? _mediaOptions.clone() : null;
+	}
+
+	public void setMediaOptions(MediaOptions options) {
+		if (_mediaSource != null) {
+			_mediaOptions = _mediaSource.validateOptions(options);
+
+		} else if (_mediaInput != null) {
+			// プレースホルダが使用されている場合
+			throw new UnsupportedOperationException();
+
+		} else {
+			// デコード時
+			_mediaOptions = (options != null) ? options.clone() : null;
+		}
 	}
 
 	@JSONHint(ignore=true)
@@ -110,18 +133,28 @@ public class MediaFileItem extends AbstractItem implements MediaItem, FileItem {
 		return _mediaInput;
 	}
 
+	public VideoBounds getPlaceholderBounds() {
+		_placeholderBounds = (_mediaSource != null && _mediaSource.isVideoAvailable())
+					? _mediaSource.getVideoFrameBounds(_mediaOptions) : null;
+		return _placeholderBounds;
+	}
+
+	public void setPlaceholderBounds(VideoBounds placeholderBounds) {
+		_placeholderBounds = placeholderBounds;
+	}
+
 	@Override
 	public void afterDecode(Project p) throws ProjectDecodeException {
 		super.afterDecode(p);
 
-		MediaFileInput input = _mediaFileInputFactory.get(getId(), _file);
+		_mediaSource = _mediaSourceFactory.get(getId(), _file);
 
-		if (input != null) {
-			_videoFrameBounds = input.isVideoAvailable() ? input.getVideoFrameBounds() : null;
-			_mediaInput = input;
+		if (_mediaSource != null) {
+			_mediaOptions = _mediaSource.validateOptions(_mediaOptions);
+			_mediaInput = new MediaInputImpl();
 		} else {
 			MediaInputPlaceholderImpl placeholder = _injector.getInstance(MediaInputPlaceholderImpl.class);
-			placeholder.setVideoFrameBounds(_videoFrameBounds);
+			placeholder.setVideoFrameBounds(_placeholderBounds);
 			_mediaInput = placeholder;
 		}
 	}
@@ -132,4 +165,36 @@ public class MediaFileItem extends AbstractItem implements MediaItem, FileItem {
 		return null;
 	}
 
+	private class MediaInputImpl implements MediaInput {
+
+		public boolean isVideoAvailable() {
+			return _mediaSource.isVideoAvailable();
+		}
+
+		public boolean isAudioAvailable() {
+			return _mediaSource.isAudioAvailable();
+		}
+
+		public Time getDuration() {
+			return _mediaSource.getDuration(_mediaOptions);
+		}
+
+		public Time getVideoFrameDuration() {
+			return _mediaSource.getVideoFrameDuration(_mediaOptions);
+		}
+
+		public VideoBounds getVideoFrameBounds() {
+			return _mediaSource.getVideoFrameBounds(_mediaOptions);
+		}
+
+		public IVideoBuffer getVideoFrame(Time mediaTime) {
+			return _mediaSource.getVideoFrame(mediaTime, _mediaOptions);
+		}
+
+		public IAudioBuffer getAudioChunk(Time mediaTime) {
+			return _mediaSource.getAudioChunk(mediaTime, _mediaOptions);
+		}
+
+	}
+
 }
