@@ -26,6 +26,7 @@ package ch.kuramo.javie.core.internal;
 import java.util.Collections;
 
 import net.arnx.jsonic.JSONHint;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.core.Camera;
@@ -43,7 +44,6 @@ import ch.kuramo.javie.core.MediaItemLayer;
 import ch.kuramo.javie.core.MotionBlur;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.VideoLayerBuffer;
 import ch.kuramo.javie.core.VideoLayerComposer;
 import ch.kuramo.javie.core.VideoLayerRenderer;
@@ -62,6 +62,10 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 
 	private MediaItem _mediaItem;
 
+	private boolean _videoEnabled;
+
+	private boolean _audioEnabled;
+
 	private boolean _ctcr;
 
 	private FrameBlend _frameBlend = FrameBlend.NONE;
@@ -77,7 +81,7 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 
 
 	@Override
-	protected void initialize(boolean videoAvailable, boolean audioAvailable) {
+	public void initialize() {
 		throw new UnsupportedOperationException("Use initialize(MediaItem) method instead.");
 	}
 
@@ -90,10 +94,12 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 			throw new IllegalArgumentException("No video nor audio is available.");
 		}
 
-		super.initialize(videoAvailable, audioAvailable);
+		super.initialize();
 
 		_itemId = mediaItem.getId();
 		_mediaItem = mediaItem;
+		_videoEnabled = videoAvailable;
+		_audioEnabled = audioAvailable;
 
 		setName(mediaItem.getName());
 	}
@@ -111,6 +117,28 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 		return _mediaItem;
 	}
 
+	@JSONHint(ignore=true)
+	public void setItem(MediaItem item) {
+		_itemId = item.getId();
+		_mediaItem = item;
+	}
+
+	public boolean isVideoEnabled() {
+		return _videoEnabled;
+	}
+
+	public void setVideoEnabled(boolean enabled) {
+		_videoEnabled = enabled;
+	}
+
+	public boolean isAudioEnabled() {
+		return _audioEnabled;
+	}
+
+	public void setAudioEnabled(boolean enabled) {
+		_audioEnabled = enabled;
+	}
+
 	public boolean isCTCR() {
 		return _ctcr;
 	}
@@ -173,23 +201,23 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 
 	@Override
 	public void setupVideoRenderer(
-			VideoLayerComposer composer, Camera camera, CollapseTransformation ct,
+			VideoLayerComposer composer, CollapseTransformation ct,
 			MotionBlur motinBlur, boolean frameBlendEnabled,
 			boolean withEffects, boolean withTransformation) {
 
 		if (LayerNature.isCTCR(this)) {
 			LayerComposition preComp = getPrecomposition();
 			if (preComp != null) {
-				setupCollapseTransformation(composer, camera, ct, motinBlur, withEffects, preComp);
+				setupCollapseTransformation(composer, ct, motinBlur, withEffects, preComp);
 				return;
 			}
 		}
 
-		super.setupVideoRenderer(composer, camera, ct, motinBlur, frameBlendEnabled, withEffects, withTransformation);
+		super.setupVideoRenderer(composer, ct, motinBlur, frameBlendEnabled, withEffects, withTransformation);
 	}
 
 	private void setupCollapseTransformation(
-			final VideoLayerComposer composer, final Camera camera,
+			final VideoLayerComposer composer,
 			final CollapseTransformation ct, final MotionBlur motionBlur,
 			boolean withEffects, final LayerComposition preComp) {
 
@@ -213,11 +241,13 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 			_context.saveAndExecute(new WrappedOperation<Object>() {
 				public Object execute() {
 					_context.setTime(calcVideoMediaTime(getMediaInput()));
-					preComp.setupCollapseTransformation(composer, camera, ct2, motionBlur);
+					preComp.setupCollapseTransformation(composer, ct2, motionBlur);
 					return null;
 				}
 			});
 		} else {
+			final Camera camera = _context.getCamera();
+
 			final VideoLayerComposerImpl composer2 = new VideoLayerComposerImpl(
 					_context, _vrSupport, _context.getColorMode(),
 					camera.getViewportSize(), camera.getPosition());
@@ -225,15 +255,15 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 			_context.saveAndExecute(new WrappedOperation<Object>() {
 				public Object execute() {
 					_context.setTime(calcVideoMediaTime(getMediaInput()));
-					preComp.setupCollapseTransformation(composer2, camera, ct2, motionBlur);
+					preComp.setupCollapseTransformation(composer2, ct2, motionBlur);
 					return null;
 				}
 			});
 
 			composer.add2D(new VideoLayerRenderer() {
 				public VideoLayerBuffer render(boolean withDepthBuffer) {
-					VideoBuffer effectedBuffer = null;
-					VideoBuffer resultBuffer = null;
+					IVideoBuffer effectedBuffer = null;
+					IVideoBuffer resultBuffer = null;
 					try {
 						WrappedOperation<VideoBounds> inputBoundsOperation = new WrappedOperation<VideoBounds>() {
 							public VideoBounds execute() {
@@ -241,9 +271,9 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 							}
 						};
 
-						WrappedOperation<VideoBuffer> inputBufferOperation = new WrappedOperation<VideoBuffer>() {
-							public VideoBuffer execute() {
-								VideoBuffer vb = composer2.compose();
+						WrappedOperation<IVideoBuffer> inputBufferOperation = new WrappedOperation<IVideoBuffer>() {
+							public IVideoBuffer execute() {
+								IVideoBuffer vb = composer2.compose();
 								_vrSupport.premultiply(vb);
 								return vb;
 							}
@@ -259,7 +289,6 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 							effectedBuffer = null;
 						} else {
 							resultBuffer = _vrSupport.createVideoBuffer(_context.getColorMode(), resultBounds);
-							resultBuffer.allocateAsTexture();
 							resultBuffer.clear();
 							_vrSupport.copy(effectedBuffer, resultBuffer);
 						}
