@@ -26,15 +26,17 @@ package ch.kuramo.javie.core.internal;
 import net.arnx.jsonic.JSONHint;
 import ch.kuramo.javie.api.Color;
 import ch.kuramo.javie.api.ColorMode;
+import ch.kuramo.javie.api.IAudioBuffer;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.core.AbstractItem;
-import ch.kuramo.javie.core.AudioBuffer;
 import ch.kuramo.javie.core.MediaInput;
+import ch.kuramo.javie.core.MediaOptions;
+import ch.kuramo.javie.core.MediaSource;
 import ch.kuramo.javie.core.SolidColorItem;
 import ch.kuramo.javie.core.VectorMediaInput;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.annotations.ProjectElement;
 import ch.kuramo.javie.core.services.RenderContext;
 import ch.kuramo.javie.core.services.VideoRenderSupport;
@@ -90,6 +92,21 @@ public class SolidColorItemImpl extends AbstractItem implements SolidColorItem {
 		_bounds = new VideoBounds(size.width, size.height);
 	}
 
+	@JSONHint(ignore=true)
+	public MediaSource getMediaSource() {
+		return null;
+	}
+
+	@JSONHint(ignore=true)
+	public MediaOptions getMediaOptions() {
+		return null;
+	}
+
+	@JSONHint(ignore=true)
+	public void setMediaOptions(MediaOptions options) {
+		throw new UnsupportedOperationException();
+	}
+
 	@JSONHint(ignore=true)
 	public MediaInput getMediaInput() {
 		return _input;
@@ -108,10 +125,6 @@ public class SolidColorItemImpl extends AbstractItem implements SolidColorItem {
 
 	private class SolidColorInput implements VectorMediaInput {
 
-		public void dispose() {
-			// nothing to do
-		}
-
 		public boolean isVideoAvailable() {
 			return true;
 		}
@@ -132,24 +145,22 @@ public class SolidColorItemImpl extends AbstractItem implements SolidColorItem {
 			return _bounds;
 		}
 
-		public VideoBuffer getVideoFrame(Time mediaTime) {
+		public IVideoBuffer getVideoFrame(Time mediaTime) {
 			ColorMode colorMode = _context.getColorMode();
 			Size2i size = _context.getVideoResolution().scale(_size);
 
-			VideoBuffer vb = _vrSupport.createVideoBuffer(colorMode, size);
-			vb.allocateAsTexture();
-
-			_vrSupport.fill(vb, _color);
+			IVideoBuffer vb = _vrSupport.createVideoBuffer(colorMode, size);
+			vb.clear(_color);
 
 			return vb;
 		}
 
-		public void rasterize(VideoBuffer resultBuffer, double[] mvMatrix, double[] prjMatrix, Time mediaTime) {
+		public void rasterize(IVideoBuffer resultBuffer, double[] mvMatrix, double[] prjMatrix, Time mediaTime) {
 			Size2i size = _context.getVideoResolution().scale(_size);
 			_vrSupport.fillRectCR(resultBuffer, mvMatrix, prjMatrix, size, _color);
 		}
 
-		public AudioBuffer getAudioChunk(Time mediaTime) {
+		public IAudioBuffer getAudioChunk(Time mediaTime) {
 			throw new UnsupportedOperationException("audio is not available");
 		}
 
