@@ -25,6 +25,7 @@ package ch.kuramo.javie.core.internal.services;
 
 import java.util.LinkedList;
 
+import ch.kuramo.javie.api.ICamera;
 import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
@@ -80,4 +81,8 @@ public class VideoEffectContextProxy
 		return getImpl().doPreviousEffect();
 	}
 
+	public ICamera getCamera() {
+		return getImpl().getCamera();
+	}
+
 }
