@@ -23,7 +23,6 @@
 
 package ch.kuramo.javie.core.output;
 
-import java.io.File;
 import java.util.Arrays;
 import java.util.concurrent.ArrayBlockingQueue;
 import java.util.concurrent.BlockingQueue;
@@ -32,11 +31,12 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import ch.kuramo.javie.api.AudioMode;
+import ch.kuramo.javie.api.Color;
+import ch.kuramo.javie.api.IAudioBuffer;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.Time;
-import ch.kuramo.javie.core.AudioBuffer;
 import ch.kuramo.javie.core.Composition;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.misc.AtiIntelLock;
 import ch.kuramo.javie.core.services.AudioRenderSupport;
 import ch.kuramo.javie.core.services.GLGlobal;
@@ -45,7 +45,7 @@ import ch.kuramo.javie.core.services.VideoRenderSupport;
 
 import com.google.inject.Inject;
 
-public abstract class PullSourceOutput {
+public abstract class PullSourceOutput extends Output {
 
 	private static final Logger _logger = LoggerFactory.getLogger(PullSourceOutput.class);
 
@@ -94,41 +94,63 @@ public abstract class PullSourceOutput {
 	private GLGlobal _glGlobal;
 
 
-	public void doOutput(final Composition comp, final AudioMode audioMode, File file, ProgressMonitor progMon) {
-		Thread videoThread = new Thread(new Runnable() { public void run() { videoLoop(comp, audioMode); } });
-		videoThread.start();
+	public void doOutput(ProgressMonitor progMon) {
+		if (getComposition() == null) throw new IllegalStateException("composition is not set");
+		if (getFile() == null) throw new IllegalStateException("file is not set");
 
-		Thread audioThread = new Thread(new Runnable() { public void run() { audioLoop(comp, audioMode); } });
-		audioThread.start();
+		boolean videoEnabled = isVideoEnabled();
+		boolean audioEnabled = isAudioEnabled();
+
+		if (!videoEnabled && !audioEnabled) {
+			throw new IllegalStateException("neither video nor audio is enabled.");
+		}
 
 		_progressMonitor = (progMon != null) ? progMon : ProgressMonitor.NULL;
 
-		doOutput(comp, audioMode, file);
-
-		try {
-			_videoRequestQueue.put(END_VIDEO);
-		} catch (InterruptedException e) {
-			_logger.warn("unexpected interruption", e);
+		if (videoEnabled) {
+			Thread videoThread = new Thread(new Runnable() { public void run() { videoLoop(); } });
+			videoThread.start();
 		}
-		try {
-			_audioRequestQueue.put(END_AUDIO);
-		} catch (InterruptedException e) {
-			_logger.warn("unexpected interruption", e);
+
+		if (audioEnabled) {
+			Thread audioThread = new Thread(new Runnable() { public void run() { audioLoop(); } });
+			audioThread.start();
+		}
+
+		doOutput();
+
+		if (videoEnabled) {
+			try {
+				_videoRequestQueue.put(END_VIDEO);
+			} catch (InterruptedException e) {
+				_logger.warn("unexpected interruption", e);
+			}
+		}
+
+		if (audioEnabled) {
+			try {
+				_audioRequestQueue.put(END_AUDIO);
+			} catch (InterruptedException e) {
+				_logger.warn("unexpected interruption", e);
+			}
 		}
 	}
 
-	protected abstract void doOutput(Composition comp, AudioMode audioMode, File file);
+	protected abstract void doOutput();
 
 
-	protected void videoLoop(Composition comp, AudioMode audioMode) {
+	protected void videoLoop() {
 		AtiIntelLock atiIntelLock = AtiIntelLock.get(_glGlobal);
 
+		Composition comp = getComposition();
+		AudioMode audioMode = getAudioMode();
+
 		_context.activate();
 		try {
 			comp.prepareExpression(_context.createInitialExpressionScope(comp));
 
 			while (true) {
-				VideoBuffer vb = null;
+				IVideoBuffer vb = null;
 				try {
 					VideoRequest request = _videoRequestQueue.take();
 					Time time = request.time;
@@ -147,8 +169,20 @@ public abstract class PullSourceOutput {
 
 						vb = comp.renderVideoFrame();
 
-						// TODO アルファ無し／ストレートアルファ／乗算済みアルファを選択できるよにする
-						_vrSupport.premultiply(vb);
+						switch (getVideoChannels()) {
+							case RGB:
+								_vrSupport.premultiply(vb, Color.BLACK, true);
+								break;
+							case ALPHA:
+								_vrSupport.alphaToGrayscale(vb);
+								break;
+							case RGBA_STRAIGHT:
+								// nothing to do
+								break;
+							case RGBA_PREMULTIPLIED:
+								_vrSupport.premultiply(vb, getColorMatte(), false);
+								break;
+						}
 
 						// glFinishしておけば finishVideoRequest は AtiIntelLock の外でいいはず。
 						//gl.glFinish();
@@ -176,10 +210,13 @@ public abstract class PullSourceOutput {
 		_videoResponseQueue.take();
 	}
 
-	protected abstract void finishVideoRequest(VideoRequest request, VideoBuffer vb);
+	protected abstract void finishVideoRequest(VideoRequest request, IVideoBuffer vb);
 
 
-	protected void audioLoop(Composition comp, AudioMode audioMode) {
+	protected void audioLoop() {
+		Composition comp = getComposition();
+		AudioMode audioMode = getAudioMode();
+
 		long totalFrames = comp.getDuration().toFrameNumber(audioMode.sampleDuration);
 
 		_context.activate();
@@ -187,7 +224,7 @@ public abstract class PullSourceOutput {
 			comp.prepareExpression(_context.createInitialExpressionScope(comp));
 
 			while (true) {
-				AudioBuffer ab = null;
+				IAudioBuffer ab = null;
 				try {
 					AudioRequest request = _audioRequestQueue.take();
 					Time time = request.time;
@@ -226,13 +263,13 @@ public abstract class PullSourceOutput {
 		_audioResponseQueue.take();
 	}
 
-	protected abstract void finishAudioRequest(AudioRequest request, AudioBuffer ab);
+	protected abstract void finishAudioRequest(AudioRequest request, IAudioBuffer ab);
 
 
-	private AudioBuffer createAudioBufferClear() {
+	private IAudioBuffer createAudioBufferClear() {
 		// このメソッドは AudioRenderSupport にあってもいいかもしれない。
 
-		AudioBuffer ab = _arSupport.createAudioBuffer();
+		IAudioBuffer ab = _arSupport.createAudioBuffer();
 
 		switch (ab.getAudioMode().dataType) {
 			case SHORT:
