@@ -66,6 +66,9 @@ public class BlendModeShaders {
 	public static final String blend_functions = "blend_functions.frag";
 
 
+	@ShaderSource(attach="blend_functions")
+	public static final String[] NONE = createProgramSource("none");
+
 	@ShaderSource(attach="blend_functions")
 	public static final String[] NORMAL = createProgramSource("normal");
 
@@ -420,88 +423,90 @@ public class BlendModeShaders {
 
 			public String[] getSource() {
 				return new String[] {
+//					 set.contains(0)  ? "vec4 blend_none(vec4 pDst, vec4 pSrc, float opacity);" : "",
 										"vec4 blend_normal(vec4 pDst, vec4 pSrc, float opacity);",
-					 set.contains(1) ||
-					 set.contains(2)  ? "vec4 blend_dissolve(vec4 pDst, vec4 pSrc, float opacity, float seed);" : "",
+					 set.contains(2) ||
+					 set.contains(3)  ? "vec4 blend_dissolve(vec4 pDst, vec4 pSrc, float opacity, float seed);" : "",
 
-					 set.contains(3)  ? "vec4 blend_darken(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(4)  ? "vec4 blend_multiply(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(5)  ? "vec4 blend_color_burn(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(6)  ? "vec4 blend_linear_burn(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(7)  ? "vec4 blend_darker_color(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(4)  ? "vec4 blend_darken(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(5)  ? "vec4 blend_multiply(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(6)  ? "vec4 blend_color_burn(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(7)  ? "vec4 blend_linear_burn(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(8)  ? "vec4 blend_darker_color(vec4 pDst, vec4 pSrc, float opacity);" : "",
 
-					 set.contains(8)  ? "vec4 blend_add(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(9)  ? "vec4 blend_lighten(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(10) ? "vec4 blend_screen(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(11) ? "vec4 blend_color_dodge(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(12) ? "vec4 blend_linear_dodge(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(13) ? "vec4 blend_lighter_color(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(9)  ? "vec4 blend_add(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(10) ? "vec4 blend_lighten(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(11) ? "vec4 blend_screen(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(12) ? "vec4 blend_color_dodge(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(13) ? "vec4 blend_linear_dodge(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(14) ? "vec4 blend_lighter_color(vec4 pDst, vec4 pSrc, float opacity);" : "",
 
-					 set.contains(14) ? "vec4 blend_overlay(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(15) ? "vec4 blend_soft_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(16) ? "vec4 blend_hard_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(17) ? "vec4 blend_linear_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(18) ? "vec4 blend_vivid_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(19) ? "vec4 blend_pin_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(20) ? "vec4 blend_hard_mix(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(15) ? "vec4 blend_overlay(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(16) ? "vec4 blend_soft_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(17) ? "vec4 blend_hard_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(18) ? "vec4 blend_linear_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(19) ? "vec4 blend_vivid_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(20) ? "vec4 blend_pin_light(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(21) ? "vec4 blend_hard_mix(vec4 pDst, vec4 pSrc, float opacity);" : "",
 
-					 set.contains(21) ? "vec4 blend_difference(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(22) ? "vec4 blend_exclusion(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(22) ? "vec4 blend_difference(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(23) ? "vec4 blend_exclusion(vec4 pDst, vec4 pSrc, float opacity);" : "",
 
-					 set.contains(23) ? "vec4 blend_hue(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(24) ? "vec4 blend_saturation(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(25) ? "vec4 blend_color(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(26) ? "vec4 blend_luminosity(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(24) ? "vec4 blend_hue(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(25) ? "vec4 blend_saturation(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(26) ? "vec4 blend_color(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(27) ? "vec4 blend_luminosity(vec4 pDst, vec4 pSrc, float opacity);" : "",
 
-//					 set.contains(27) ? "vec4 blend_stencil_alpha(vec4 pDst, vec4 pSrc, float opacity);" : "",
-//					 set.contains(28) ? "vec4 blend_stencil_luma(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(29) ? "vec4 blend_silhouette_alpha(vec4 pDst, vec4 pSrc, float opacity);" : "",
-					 set.contains(30) ? "vec4 blend_silhouette_luma(vec4 pDst, vec4 pSrc, float opacity);" : "",
+//					 set.contains(28) ? "vec4 blend_stencil_alpha(vec4 pDst, vec4 pSrc, float opacity);" : "",
+//					 set.contains(29) ? "vec4 blend_stencil_luma(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(30) ? "vec4 blend_silhouette_alpha(vec4 pDst, vec4 pSrc, float opacity);" : "",
+					 set.contains(31) ? "vec4 blend_silhouette_luma(vec4 pDst, vec4 pSrc, float opacity);" : "",
 
 										"vec4 blend(int mode, vec4 pDst, vec4 pSrc, float opacity, float dissolveSeed)",
 										"{",
 										"	return",
-					 set.contains(0)  ? "		(mode == 0) ? blend_normal(pDst, pSrc, opacity) :" : "",
-					 set.contains(1)  ? "		(mode == 1) ? blend_dissolve(pDst, pSrc, opacity, dissolveSeed) :" : "",
+//					 set.contains(0)  ? "		(mode == 0) ? blend_none(pDst, pSrc, opacity) :" : "",
+					 set.contains(1)  ? "		(mode == 1) ? blend_normal(pDst, pSrc, opacity) :" : "",
 					 set.contains(2)  ? "		(mode == 2) ? blend_dissolve(pDst, pSrc, opacity, dissolveSeed) :" : "",
+					 set.contains(3)  ? "		(mode == 3) ? blend_dissolve(pDst, pSrc, opacity, dissolveSeed) :" : "",
 
-					 set.contains(3)  ? "		(mode == 3) ? blend_darken(pDst, pSrc, opacity) :" : "",
-					 set.contains(4)  ? "		(mode == 4) ? blend_multiply(pDst, pSrc, opacity) :" : "",
-					 set.contains(5)  ? "		(mode == 5) ? blend_color_burn(pDst, pSrc, opacity) :" : "",
-					 set.contains(6)  ? "		(mode == 6) ? blend_linear_burn(pDst, pSrc, opacity) :" : "",
-					 set.contains(7)  ? "		(mode == 7) ? blend_darker_color(pDst, pSrc, opacity) :" : "",
+					 set.contains(4)  ? "		(mode == 4) ? blend_darken(pDst, pSrc, opacity) :" : "",
+					 set.contains(5)  ? "		(mode == 5) ? blend_multiply(pDst, pSrc, opacity) :" : "",
+					 set.contains(6)  ? "		(mode == 6) ? blend_color_burn(pDst, pSrc, opacity) :" : "",
+					 set.contains(7)  ? "		(mode == 7) ? blend_linear_burn(pDst, pSrc, opacity) :" : "",
+					 set.contains(8)  ? "		(mode == 8) ? blend_darker_color(pDst, pSrc, opacity) :" : "",
 
-					 set.contains(8)  ? "		(mode == 8) ? blend_add(pDst, pSrc, opacity) :" : "",
-					 set.contains(9)  ? "		(mode == 9) ? blend_lighten(pDst, pSrc, opacity) :" : "",
-					 set.contains(10) ? "		(mode == 10) ? blend_screen(pDst, pSrc, opacity) :" : "",
-					 set.contains(11) ? "		(mode == 11) ? blend_color_dodge(pDst, pSrc, opacity) :" : "",
-					 set.contains(12) ? "		(mode == 12) ? blend_linear_dodge(pDst, pSrc, opacity) :" : "",
-					 set.contains(13) ? "		(mode == 13) ? blend_lighter_color(pDst, pSrc, opacity) :" : "",
+					 set.contains(9)  ? "		(mode == 9) ? blend_add(pDst, pSrc, opacity) :" : "",
+					 set.contains(10) ? "		(mode == 10) ? blend_lighten(pDst, pSrc, opacity) :" : "",
+					 set.contains(11) ? "		(mode == 11) ? blend_screen(pDst, pSrc, opacity) :" : "",
+					 set.contains(12) ? "		(mode == 12) ? blend_color_dodge(pDst, pSrc, opacity) :" : "",
+					 set.contains(13) ? "		(mode == 13) ? blend_linear_dodge(pDst, pSrc, opacity) :" : "",
+					 set.contains(14) ? "		(mode == 14) ? blend_lighter_color(pDst, pSrc, opacity) :" : "",
 
-					 set.contains(14) ? "		(mode == 14) ? blend_overlay(pDst, pSrc, opacity) :" : "",
-					 set.contains(15) ? "		(mode == 15) ? blend_soft_light(pDst, pSrc, opacity) :" : "",
-					 set.contains(16) ? "		(mode == 16) ? blend_hard_light(pDst, pSrc, opacity) :" : "",
-					 set.contains(17) ? "		(mode == 17) ? blend_linear_light(pDst, pSrc, opacity) :" : "",
-					 set.contains(18) ? "		(mode == 18) ? blend_vivid_light(pDst, pSrc, opacity) :" : "",
-					 set.contains(19) ? "		(mode == 19) ? blend_pin_light(pDst, pSrc, opacity) :" : "",
-					 set.contains(20) ? "		(mode == 20) ? blend_hard_mix(pDst, pSrc, opacity) :" : "",
+					 set.contains(15) ? "		(mode == 15) ? blend_overlay(pDst, pSrc, opacity) :" : "",
+					 set.contains(16) ? "		(mode == 16) ? blend_soft_light(pDst, pSrc, opacity) :" : "",
+					 set.contains(17) ? "		(mode == 17) ? blend_hard_light(pDst, pSrc, opacity) :" : "",
+					 set.contains(18) ? "		(mode == 18) ? blend_linear_light(pDst, pSrc, opacity) :" : "",
+					 set.contains(19) ? "		(mode == 19) ? blend_vivid_light(pDst, pSrc, opacity) :" : "",
+					 set.contains(20) ? "		(mode == 20) ? blend_pin_light(pDst, pSrc, opacity) :" : "",
+					 set.contains(21) ? "		(mode == 21) ? blend_hard_mix(pDst, pSrc, opacity) :" : "",
 
-					 set.contains(21) ? "		(mode == 21) ? blend_difference(pDst, pSrc, opacity) :" : "",
-					 set.contains(22) ? "		(mode == 22) ? blend_exclusion(pDst, pSrc, opacity) :" : "",
+					 set.contains(22) ? "		(mode == 22) ? blend_difference(pDst, pSrc, opacity) :" : "",
+					 set.contains(23) ? "		(mode == 23) ? blend_exclusion(pDst, pSrc, opacity) :" : "",
 
-					 set.contains(23) ? "		(mode == 23) ? blend_hue(pDst, pSrc, opacity) :" : "",
-					 set.contains(24) ? "		(mode == 24) ? blend_saturation(pDst, pSrc, opacity) :" : "",
-					 set.contains(25) ? "		(mode == 25) ? blend_color(pDst, pSrc, opacity) :" : "",
-					 set.contains(26) ? "		(mode == 26) ? blend_luminosity(pDst, pSrc, opacity) :" : "",
+					 set.contains(24) ? "		(mode == 24) ? blend_hue(pDst, pSrc, opacity) :" : "",
+					 set.contains(25) ? "		(mode == 25) ? blend_saturation(pDst, pSrc, opacity) :" : "",
+					 set.contains(26) ? "		(mode == 26) ? blend_color(pDst, pSrc, opacity) :" : "",
+					 set.contains(27) ? "		(mode == 27) ? blend_luminosity(pDst, pSrc, opacity) :" : "",
 
 //TODO ステンシルアルファとステンシルルミナンスは、次のふたつの理由で3Dレイヤーでは正しく動作しない。
 //      1. レイヤー矩形の外側はアルファ値がゼロなので、ノンゼロアルファ(nza)のテストで落とされてしまう（これはシェーダの修正だけで解決可能）
 //      2. レイヤー矩形の外側はデプス値が最大なので、ソートしたときに一番奥に配置されてしまう（これはシェーダの修正だけでは解決できない）
 //
-//					 set.contains(27) ? "		(mode == 27) ? blend_stencil_alpha(pDst, pSrc, opacity) :" : "",
-//					 set.contains(28) ? "		(mode == 28) ? blend_stencil_luma(pDst, pSrc, opacity) :" : "",
-					 set.contains(29) ? "		(mode == 29) ? blend_silhouette_alpha(pDst, pSrc, opacity) :" : "",
-					 set.contains(30) ? "		(mode == 30) ? blend_silhouette_luma(pDst, pSrc, opacity) :" : "",
+//					 set.contains(28) ? "		(mode == 28) ? blend_stencil_alpha(pDst, pSrc, opacity) :" : "",
+//					 set.contains(29) ? "		(mode == 29) ? blend_stencil_luma(pDst, pSrc, opacity) :" : "",
+					 set.contains(30) ? "		(mode == 30) ? blend_silhouette_alpha(pDst, pSrc, opacity) :" : "",
+					 set.contains(31) ? "		(mode == 31) ? blend_silhouette_luma(pDst, pSrc, opacity) :" : "",
 
 										"					   blend_normal(pDst, pSrc, opacity);",
 										"}"
