@@ -139,8 +139,8 @@
 }
 
 -(id) initWithFile:(NSString*)filename
-		cglContext:(CGLContextObj)cglContext
-	cglPixelFormat:(CGLPixelFormatObj)cglPixelFormat
+				  :(CGLContextObj)cglContext
+				  :(CGLPixelFormatObj)cglPixelFormat
 {
 	[super init];
 	
@@ -227,11 +227,10 @@
 		return FALSE;
 	}
 	
-	// QuartzComposerのファイルの場合、デュレーションを60秒、サンプル数を3600(60fps換算で60秒分)に設定する。
-	// (ムービー自体のデュレーションは、JNI_OnLoad内でQuartzComposerDefaultMovieDurationを300*60*60秒に設定してある)
 	if (mMediaType == kQTQuartzComposerMediaType) {
-		mDuration = mTimeScale * 60;
-		mSampleCount = 60 * 60;
+		// デュレーションは、JNI_OnLoad内でQuartzComposerDefaultMovieDurationを300*60*60秒に設定してある
+		mDuration = mTimeScale * 300*60*60;
+		mSampleCount = 0;
 	}
 	
 	return TRUE;
@@ -301,9 +300,14 @@
 }
 
 -(OSStatus) frameImageAtTime:(QTTime)time
-					 texture:(GLint)texture
-					   width:(GLint)width
-					  height:(GLint)height
+							:(GLint)texture
+							:(GLint)width
+							:(GLint)height
+							:(int)alphaType
+							:(float)colorMatteR
+							:(float)colorMatteG
+							:(float)colorMatteB
+							:(BOOL)flipVertical
 {
 	if (width != mTargetWidth || height != mTargetHeight) {
 		mTargetWidth = width;
@@ -335,50 +339,82 @@
 		return error;
 	}
 	
-	
-	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_EXT, texture, 0);
-	glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
-	
-	glClearColor(0, 0, 0, 0);
-	glClear(GL_COLOR_BUFFER_BIT);
-	
-	
 	GLenum target = CVOpenGLTextureGetTarget(mCurrentFrame);
 	GLint name = CVOpenGLTextureGetName(mCurrentFrame);
 	
 	GLfloat topLeft[2], topRight[2], bottomRight[2], bottomLeft[2];
 	CVOpenGLTextureGetCleanTexCoords(mCurrentFrame, bottomLeft, bottomRight, topRight, topLeft);
 	
+	
+	glPushAttrib(GL_COLOR_BUFFER_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT | GL_CURRENT_BIT);
+	
+	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_EXT, texture, 0);
+	glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
+	
+	switch (alphaType) {
+		// AlphaType.IGNORE
+		case 0:
+			glClearColor(0, 0, 0, 1);
+			glClear(GL_COLOR_BUFFER_BIT);
+			
+			glEnable(GL_BLEND);
+			glBlendFuncSeparate(GL_ONE, GL_ZERO, GL_ZERO, GL_ONE);
+			glBlendEquation(GL_FUNC_ADD);
+			break;
+			
+		// AlphaType.STRAIGHT
+		case 1:
+			glEnable(GL_BLEND);
+			glBlendFuncSeparate(GL_SRC_ALPHA, GL_ZERO, GL_ONE, GL_ZERO);
+			glBlendEquation(GL_FUNC_ADD);
+			break;
+			
+		// AlphaType.PREMULTIPLIED
+		case 2:
+			if (colorMatteR == 0 && colorMatteG == 0 && colorMatteB == 0) {
+				glDisable(GL_BLEND);
+
+			} else {
+				glClearColor(colorMatteR, colorMatteG, colorMatteB, 1);
+				glClear(GL_COLOR_BUFFER_BIT);
+				
+				glEnable(GL_BLEND);
+				glBlendFuncSeparate(GL_ONE, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_ZERO);
+				glBlendEquation(GL_FUNC_SUBTRACT);
+			}
+			break;
+	}
+	
 	glViewport(0, 0, width, height);
 	
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-	gluOrtho2D(0, width, 0, height);
+	if (flipVertical) {
+		gluOrtho2D(0, width, height, 0);
+	} else {
+		gluOrtho2D(0, width, 0, height);
+	}
 	
-	//glMatrixMode(GL_MODELVIEW);
-	//glLoadIdentity();
-	
-	//glMatrixMode(GL_TEXTURE);
-	//glLoadIdentity();
+	glMatrixMode(GL_MODELVIEW);
+	glLoadIdentity();
 	
 	glBindTexture(target, name);
 	glEnable(target);
-	glBegin(GL_QUADS);
 	
-	//glColor4d(1, 1, 1, 1);
+	glColor4d(1, 1, 1, 1);
+	
+	glBegin(GL_QUADS);
 	glTexCoord2fv(topLeft);     glVertex2i(0, 0);
 	glTexCoord2fv(topRight);    glVertex2i(width, 0);
 	glTexCoord2fv(bottomRight); glVertex2i(width, height);
 	glTexCoord2fv(bottomLeft);  glVertex2i(0, height);
-	
 	glEnd();
-	glDisable(target);
-	glBindTexture(target, 0);
 	
-	
-	glFlush();
+	glFinish();
 	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_EXT, 0, 0);
 	
+	glPopAttrib();
+	
 	QTVisualContextTask(mVisualContext);
 	
 	return noErr;
@@ -508,9 +544,9 @@
 }
 
 -(OSStatus) audioChunkFromTime:(QTTime)time
-						buffer:(void*)buffer
-						offset:(long)offset
-					frameCount:(long)frameCount
+							  :(void*)buffer
+							  :(long)offset
+							  :(long)frameCount
 {
 	if (mAudioExtraction == NULL) {
 		NSLog(@"AudioExtraction is not initialized");
