@@ -23,7 +23,11 @@
 
 package ch.kuramo.javie.effects.colorCorrection;
 
-import javax.media.opengl.GL2;
+import java.nio.FloatBuffer;
+import java.util.HashSet;
+import java.util.Set;
+
+import javax.media.opengl.GLUniformData;
 
 import ch.kuramo.javie.api.Color;
 import ch.kuramo.javie.api.IAnimatableColor;
@@ -34,13 +38,14 @@ import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.api.annotations.Effect;
 import ch.kuramo.javie.api.annotations.Property;
 import ch.kuramo.javie.api.annotations.ShaderSource;
+import ch.kuramo.javie.api.annotations.Effect.Categories;
 import ch.kuramo.javie.api.services.IShaderRegistry;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
-import ch.kuramo.javie.effects.VideoEffectUtil;
+import ch.kuramo.javie.api.services.IVideoRenderSupport;
 
 import com.google.inject.Inject;
 
-@Effect(id="ch.kuramo.javie.Tritone", category=Effect.COLOR_CORRECTION)
+@Effect(id="ch.kuramo.javie.Tritone", category=Categories.COLOR_CORRECTION)
 public class Tritone {
 
 	@ShaderSource
@@ -73,7 +78,9 @@ public class Tritone {
 
 	private final IVideoEffectContext context;
 
-	private final IShaderProgram tritoneProgram;
+	private final IVideoRenderSupport support;
+
+	private final IShaderProgram program;
 
 	@Property("1,1,1")
 	private IAnimatableColor highlight;
@@ -89,83 +96,46 @@ public class Tritone {
 
 
 	@Inject
-	public Tritone(IVideoEffectContext context, IShaderRegistry shaders) {
+	public Tritone(IVideoEffectContext context, IVideoRenderSupport support, IShaderRegistry shaders) {
 		this.context = context;
-		tritoneProgram = shaders.getProgram(Tritone.class, "TRITONE");
+		this.support = support;
+		program = shaders.getProgram(Tritone.class, "TRITONE");
 	}
 
 	public IVideoBuffer doVideoEffect() {
-		final double blend = context.value(blendWithOriginal)/100;
-		if (blend == 1) {
-			return null;
+		IVideoBuffer input = context.doPreviousEffect();
+
+		VideoBounds bounds = input.getBounds();
+		if (bounds.isEmpty()) {
+			return input;
 		}
 
-		final Color highlight = context.value(this.highlight);
-		final Color midtone = context.value(this.midtone);
-		final Color shadow = context.value(this.shadow);
+		double blend = context.value(blendWithOriginal);
+		if (blend == 100) {
+			return input;
+		}
+
+		Color highlight = context.value(this.highlight).clamp();
+		Color midtone = context.value(this.midtone).clamp();
+		Color shadow = context.value(this.shadow).clamp();
 
-		IVideoBuffer input = null;
-		IVideoBuffer buf = null;
 		try {
-			input = context.doPreviousEffect();
-			VideoBounds bounds = input.getBounds();
-			if (bounds.isEmpty()) {
-				IVideoBuffer output = input;
-				input = null;
-				return output;
-			}
+			Set<GLUniformData> uniforms = new HashSet<GLUniformData>();
+			uniforms.add(new GLUniformData("texture", 0));
+			uniforms.add(new GLUniformData("highlight", 3, FloatBuffer.wrap(colorToFloat3(highlight))));
+			uniforms.add(new GLUniformData("midtone", 3, FloatBuffer.wrap(colorToFloat3(midtone))));
+			uniforms.add(new GLUniformData("shadow", 3, FloatBuffer.wrap(colorToFloat3(shadow))));
+			uniforms.add(new GLUniformData("blend", (float)(blend/100)));
 
-			final int w = bounds.width;
-			final int h = bounds.height;
-
-			buf = context.createVideoBuffer(bounds);
-
-			final GL2 gl = context.getGL().getGL2();
-
-			VideoEffectUtil.ortho2D(gl, context.getGLU(), w, h);
-
-			gl.glFramebufferTexture2D(GL2.GL_FRAMEBUFFER,
-					GL2.GL_COLOR_ATTACHMENT0, GL2.GL_TEXTURE_RECTANGLE, buf.getTexture(), 0);
-			gl.glDrawBuffer(GL2.GL_COLOR_ATTACHMENT0);
-
-			gl.glActiveTexture(GL2.GL_TEXTURE0);
-			gl.glBindTexture(GL2.GL_TEXTURE_RECTANGLE, input.getTexture());
-
-			tritoneProgram.useProgram(new Runnable() {
-				public void run() {
-					gl.glUniform1i(tritoneProgram.getUniformLocation("texture"), 0);
-					gl.glUniform3f(tritoneProgram.getUniformLocation("highlight"), (float)highlight.r, (float)highlight.g, (float)highlight.b);
-					gl.glUniform3f(tritoneProgram.getUniformLocation("midtone"), (float)midtone.r, (float)midtone.g, (float)midtone.b);
-					gl.glUniform3f(tritoneProgram.getUniformLocation("shadow"), (float)shadow.r, (float)shadow.g, (float)shadow.b);
-					gl.glUniform1f(tritoneProgram.getUniformLocation("blend"), (float) blend);
-
-					gl.glBegin(GL2.GL_QUADS);
-					gl.glTexCoord2f(0, 0);
-					gl.glVertex2f(0, 0);
-					gl.glTexCoord2f(w, 0);
-					gl.glVertex2f(w, 0);
-					gl.glTexCoord2f(w, h);
-					gl.glVertex2f(w, h);
-					gl.glTexCoord2f(0, h);
-					gl.glVertex2f(0, h);
-					gl.glEnd();
-				}
-			});
-
-			gl.glActiveTexture(GL2.GL_TEXTURE0);
-			gl.glBindTexture(GL2.GL_TEXTURE_RECTANGLE, 0);
-
-			gl.glFramebufferTexture2D(GL2.GL_FRAMEBUFFER,
-					GL2.GL_COLOR_ATTACHMENT0, GL2.GL_TEXTURE_RECTANGLE, 0, 0);
-
-			IVideoBuffer output = buf;
-			buf = null;
-			return output;
+			return support.useShaderProgram(program, uniforms, null, input);
 
 		} finally {
-			if (input != null) input.dispose();
-			if (buf != null) buf.dispose();
+			input.dispose();
 		}
 	}
 
+	private float[] colorToFloat3(Color color) {
+		return new float[] { (float)color.r, (float)color.g, (float)color.b };
+	}
+
 }
