@@ -260,7 +260,7 @@ public class Shatter {
 
 		double patDirRad = Math.toRadians(patDir);
 
-		double[] b2 = rotate(bounds, patOrigin, patDirRad);
+		double[] b2 = rotate(bounds, patOrigin, -patDirRad);
 		int i1 = (int)Math.ceil((b2[0]-(patOrigin.x+fragSize.x/2))/fragSize.x);
 		int j1 = (int)Math.ceil((b2[1]-(patOrigin.y+fragSize.y/2))/fragSize.y);
 		int i2 = (int)Math.ceil((b2[2]-(patOrigin.x+fragSize.x/2))/fragSize.x);
@@ -512,8 +512,8 @@ public class Shatter {
 				{ bounds.x             , bounds.y+bounds.height },
 		};
 
-		double cos = Math.cos(-rad);
-		double sin = Math.sin(-rad);
+		double cos = Math.cos(rad);
+		double sin = Math.sin(rad);
 
 		double[] result = new double[] {
 			Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY,
